@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -29,7 +29,6 @@ import java.util.Properties;
 
 import junit.framework.Assert;
 import junit.framework.TestCase;
-
 import org.apache.avro.Schema;
 import org.apache.avro.file.DataFileWriter;
 import org.apache.avro.generic.GenericData;
@@ -42,6 +41,7 @@ import org.apache.avro.specific.SpecificDatumWriter;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FSDataOutputStream;
 import org.apache.hadoop.fs.Path;
+import org.apache.hadoop.hbase.Cell;
 import org.apache.hadoop.hbase.KeyValue;
 import org.apache.hadoop.hbase.client.Put;
 import org.apache.hadoop.hbase.client.Result;
@@ -56,13 +56,13 @@ import org.apache.hadoop.hive.hbase.avro.OfficePhone;
 import org.apache.hadoop.hive.serde.serdeConstants;
 import org.apache.hadoop.hive.serde2.SerDeException;
 import org.apache.hadoop.hive.serde2.SerDeUtils;
-import org.apache.hadoop.hive.serde2.avro.AvroSerdeUtils;
+import org.apache.hadoop.hive.serde2.avro.AvroSerdeUtils.AvroTableProperties;
 import org.apache.hadoop.hive.serde2.io.ByteWritable;
 import org.apache.hadoop.hive.serde2.io.DoubleWritable;
 import org.apache.hadoop.hive.serde2.io.ShortWritable;
 import org.apache.hadoop.hive.serde2.lazy.LazyPrimitive;
-import org.apache.hadoop.hive.serde2.lazy.LazyStruct;
 import org.apache.hadoop.hive.serde2.lazy.LazySerDeParameters;
+import org.apache.hadoop.hive.serde2.lazy.LazyStruct;
 import org.apache.hadoop.hive.serde2.objectinspector.StructField;
 import org.apache.hadoop.hive.serde2.objectinspector.StructObjectInspector;
 import org.apache.hadoop.io.BooleanWritable;
@@ -179,7 +179,7 @@ public class TestHBaseSerDe extends TestCase {
     byte [] rowKey = Bytes.toBytes("test-row1");
 
     // Data
-    List<KeyValue> kvs = new ArrayList<KeyValue>();
+    List<Cell> kvs = new ArrayList<Cell>();
 
     kvs.add(new KeyValue(rowKey, cfa, qualByte, Bytes.toBytes("123")));
     kvs.add(new KeyValue(rowKey, cfb, qualShort, Bytes.toBytes("456")));
@@ -191,18 +191,18 @@ public class TestHBaseSerDe extends TestCase {
     kvs.add(new KeyValue(rowKey, cfb, qualBool, Bytes.toBytes("true")));
     Collections.sort(kvs, KeyValue.COMPARATOR);
 
-    Result r = new Result(kvs);
+    Result r = Result.create(kvs);
 
     Put p = new Put(rowKey);
 
-    p.add(cfa, qualByte, Bytes.toBytes("123"));
-    p.add(cfb, qualShort, Bytes.toBytes("456"));
-    p.add(cfc, qualInt, Bytes.toBytes("789"));
-    p.add(cfa, qualLong, Bytes.toBytes("1000"));
-    p.add(cfb, qualFloat, Bytes.toBytes("-0.01"));
-    p.add(cfc, qualDouble, Bytes.toBytes("5.3"));
-    p.add(cfa, qualString, Bytes.toBytes("Hadoop, HBase, and Hive"));
-    p.add(cfb, qualBool, Bytes.toBytes("true"));
+    p.addColumn(cfa, qualByte, Bytes.toBytes("123"));
+    p.addColumn(cfb, qualShort, Bytes.toBytes("456"));
+    p.addColumn(cfc, qualInt, Bytes.toBytes("789"));
+    p.addColumn(cfa, qualLong, Bytes.toBytes("1000"));
+    p.addColumn(cfb, qualFloat, Bytes.toBytes("-0.01"));
+    p.addColumn(cfc, qualDouble, Bytes.toBytes("5.3"));
+    p.addColumn(cfa, qualString, Bytes.toBytes("Hadoop, HBase, and Hive"));
+    p.addColumn(cfb, qualBool, Bytes.toBytes("true"));
 
     Object[] expectedFieldsData = {
       new Text("test-row1"),
@@ -273,7 +273,7 @@ public class TestHBaseSerDe extends TestCase {
     byte [] rowKey = Bytes.toBytes("test-row1");
 
     // Data
-    List<KeyValue> kvs = new ArrayList<KeyValue>();
+    List<Cell> kvs = new ArrayList<Cell>();
 
     kvs.add(new KeyValue(rowKey, cfa, qualByte, Bytes.toBytes("123")));
     kvs.add(new KeyValue(rowKey, cfb, qualShort, Bytes.toBytes("456")));
@@ -285,18 +285,18 @@ public class TestHBaseSerDe extends TestCase {
     kvs.add(new KeyValue(rowKey, cfb, qualBool, Bytes.toBytes("true")));
     Collections.sort(kvs, KeyValue.COMPARATOR);
 
-    Result r = new Result(kvs);
+    Result r = Result.create(kvs);
 
     Put p = new Put(rowKey,putTimestamp);
 
-    p.add(cfa, qualByte, Bytes.toBytes("123"));
-    p.add(cfb, qualShort, Bytes.toBytes("456"));
-    p.add(cfc, qualInt, Bytes.toBytes("789"));
-    p.add(cfa, qualLong, Bytes.toBytes("1000"));
-    p.add(cfb, qualFloat, Bytes.toBytes("-0.01"));
-    p.add(cfc, qualDouble, Bytes.toBytes("5.3"));
-    p.add(cfa, qualString, Bytes.toBytes("Hadoop, HBase, and Hive"));
-    p.add(cfb, qualBool, Bytes.toBytes("true"));
+    p.addColumn(cfa, qualByte, Bytes.toBytes("123"));
+    p.addColumn(cfb, qualShort, Bytes.toBytes("456"));
+    p.addColumn(cfc, qualInt, Bytes.toBytes("789"));
+    p.addColumn(cfa, qualLong, Bytes.toBytes("1000"));
+    p.addColumn(cfb, qualFloat, Bytes.toBytes("-0.01"));
+    p.addColumn(cfc, qualDouble, Bytes.toBytes("5.3"));
+    p.addColumn(cfa, qualString, Bytes.toBytes("Hadoop, HBase, and Hive"));
+    p.addColumn(cfb, qualBool, Bytes.toBytes("true"));
 
     Object[] expectedFieldsData = {
       new Text("test-row1"),
@@ -419,7 +419,7 @@ public class TestHBaseSerDe extends TestCase {
     byte [] rowKey = Bytes.toBytes("test-row-2");
 
     // Data
-    List<KeyValue> kvs = new ArrayList<KeyValue>();
+    List<Cell> kvs = new ArrayList<Cell>();
 
     kvs.add(new KeyValue(rowKey, cfa, qualByte, new byte [] { Byte.MIN_VALUE }));
     kvs.add(new KeyValue(rowKey, cfb, qualShort, Bytes.toBytes(Short.MIN_VALUE)));
@@ -431,19 +431,21 @@ public class TestHBaseSerDe extends TestCase {
       "Hadoop, HBase, and Hive Again!")));
     kvs.add(new KeyValue(rowKey, cfb, qualBool, Bytes.toBytes(false)));
 
+//    When using only HBase2, then we could change to this
+//    Collections.sort(kvs, CellComparator.COMPARATOR);
     Collections.sort(kvs, KeyValue.COMPARATOR);
-    Result r = new Result(kvs);
+    Result r = Result.create(kvs);
 
     Put p = new Put(rowKey);
 
-    p.add(cfa, qualByte, new byte [] { Byte.MIN_VALUE });
-    p.add(cfb, qualShort, Bytes.toBytes(Short.MIN_VALUE));
-    p.add(cfc, qualInt, Bytes.toBytes(Integer.MIN_VALUE));
-    p.add(cfa, qualLong, Bytes.toBytes(Long.MIN_VALUE));
-    p.add(cfb, qualFloat, Bytes.toBytes(Float.MIN_VALUE));
-    p.add(cfc, qualDouble, Bytes.toBytes(Double.MAX_VALUE));
-    p.add(cfa, qualString, Bytes.toBytes("Hadoop, HBase, and Hive Again!"));
-    p.add(cfb, qualBool, Bytes.toBytes(false));
+    p.addColumn(cfa, qualByte, new byte [] { Byte.MIN_VALUE });
+    p.addColumn(cfb, qualShort, Bytes.toBytes(Short.MIN_VALUE));
+    p.addColumn(cfc, qualInt, Bytes.toBytes(Integer.MIN_VALUE));
+    p.addColumn(cfa, qualLong, Bytes.toBytes(Long.MIN_VALUE));
+    p.addColumn(cfb, qualFloat, Bytes.toBytes(Float.MIN_VALUE));
+    p.addColumn(cfc, qualDouble, Bytes.toBytes(Double.MAX_VALUE));
+    p.addColumn(cfa, qualString, Bytes.toBytes("Hadoop, HBase, and Hive Again!"));
+    p.addColumn(cfb, qualBool, Bytes.toBytes(false));
 
     Object[] expectedFieldsData = {
       new Text("test-row-2"),
@@ -557,7 +559,7 @@ public class TestHBaseSerDe extends TestCase {
          Bytes.toBytes(true)}
     };
 
-    List<KeyValue> kvs = new ArrayList<KeyValue>();
+    List<Cell> kvs = new ArrayList<Cell>();
     Result [] r = new Result [] {null, null, null};
     Put [] p = new Put [] {null, null, null};
 
@@ -568,11 +570,11 @@ public class TestHBaseSerDe extends TestCase {
       for (int j = 0; j < columnQualifiersAndValues[i].length; j++) {
         kvs.add(new KeyValue(rowKeys[i], columnFamilies[j], columnQualifiersAndValues[i][j],
             columnQualifiersAndValues[i][j]));
-        p[i].add(columnFamilies[j], columnQualifiersAndValues[i][j],
+        p[i].addColumn(columnFamilies[j], columnQualifiersAndValues[i][j],
             columnQualifiersAndValues[i][j]);
       }
 
-      r[i] = new Result(kvs);
+      r[i] = Result.create(kvs);
     }
 
     Object [][] expectedData = {
@@ -701,15 +703,15 @@ public class TestHBaseSerDe extends TestCase {
     };
 
     Put p = new Put(rowKey);
-    List<KeyValue> kvs = new ArrayList<KeyValue>();
+    List<Cell> kvs = new ArrayList<Cell>();
 
     for (int j = 0; j < columnQualifiersAndValues.length; j++) {
       kvs.add(new KeyValue(rowKey,
           columnFamilies[j], columnQualifiersAndValues[j], columnQualifiersAndValues[j]));
-      p.add(columnFamilies[j], columnQualifiersAndValues[j], columnQualifiersAndValues[j]);
+      p.addColumn(columnFamilies[j], columnQualifiersAndValues[j], columnQualifiersAndValues[j]);
     }
 
-    Result r = new Result(kvs);
+    Result r = Result.create(kvs);
 
     Object [] expectedData = {
         new Text("row-key"), new ByteWritable((byte) 123), new ShortWritable((short) 456),
@@ -821,7 +823,7 @@ public class TestHBaseSerDe extends TestCase {
     byte[] rowKey = Bytes.toBytes("test-row1");
 
     // Data
-    List<KeyValue> kvs = new ArrayList<KeyValue>();
+    List<Cell> kvs = new ArrayList<Cell>();
 
     byte[] dataA = "This is first test data".getBytes();
     byte[] dataB = "This is second test data".getBytes();
@@ -833,7 +835,7 @@ public class TestHBaseSerDe extends TestCase {
     kvs.add(new KeyValue(rowKey, cfa, qualC, dataC));
     kvs.add(new KeyValue(rowKey, cfa, qualD, dataD));
 
-    Result r = new Result(kvs);
+    Result r = Result.create(kvs);
 
     Put p = new Put(rowKey);
 
@@ -928,13 +930,13 @@ public class TestHBaseSerDe extends TestCase {
     byte[] rowKey = testStruct.getBytes();
 
     // Data
-    List<KeyValue> kvs = new ArrayList<KeyValue>();
+    List<Cell> kvs = new ArrayList<Cell>();
 
     byte[] testData = "This is a test data".getBytes();
 
     kvs.add(new KeyValue(rowKey, cfa, qualStruct, testData));
 
-    Result r = new Result(kvs);
+    Result r = Result.create(kvs);
 
     Put p = new Put(rowKey);
 
@@ -976,13 +978,13 @@ public class TestHBaseSerDe extends TestCase {
     byte[] rowKey = testStruct.getBytes();
 
     // Data
-    List<KeyValue> kvs = new ArrayList<KeyValue>();
+    List<Cell> kvs = new ArrayList<Cell>();
 
     byte[] testData = "This is a test data".getBytes();
 
     kvs.add(new KeyValue(rowKey, cfa, qualStruct, testData));
 
-    Result r = new Result(kvs);
+    Result r = Result.create(kvs);
 
     byte[] putRowKey = testStruct.getBytesWithDelimiters();
 
@@ -1047,13 +1049,13 @@ public class TestHBaseSerDe extends TestCase {
     byte[] rowKey = Bytes.toBytes("test-row1");
 
     // Data
-    List<KeyValue> kvs = new ArrayList<KeyValue>();
+    List<Cell> kvs = new ArrayList<Cell>();
 
     byte[] avroData = getTestAvroBytesFromSchema(RECORD_SCHEMA);
 
     kvs.add(new KeyValue(rowKey, cfa, qualAvro, avroData));
 
-    Result r = new Result(kvs);
+    Result r = Result.create(kvs);
 
     Put p = new Put(rowKey);
 
@@ -1077,7 +1079,7 @@ public class TestHBaseSerDe extends TestCase {
   private Properties createPropertiesForHiveAvroSchemaInline() {
     Properties tbl = new Properties();
     tbl.setProperty("cola.avro.serialization.type", "avro");
-    tbl.setProperty("cola.avro." + AvroSerdeUtils.SCHEMA_LITERAL, RECORD_SCHEMA);
+    tbl.setProperty("cola.avro." + AvroTableProperties.SCHEMA_LITERAL.getPropName(), RECORD_SCHEMA);
     tbl.setProperty(HBaseSerDe.HBASE_COLUMNS_MAPPING, ":key,cola:avro");
     tbl.setProperty(HBaseSerDe.HBASE_AUTOGENERATE_STRUCT, "true");
 
@@ -1092,13 +1094,13 @@ public class TestHBaseSerDe extends TestCase {
     byte[] rowKey = Bytes.toBytes("test-row1");
 
     // Data
-    List<KeyValue> kvs = new ArrayList<KeyValue>();
+    List<Cell> kvs = new ArrayList<Cell>();
 
     byte[] avroData = getTestAvroBytesFromSchema(RECORD_SCHEMA);
 
     kvs.add(new KeyValue(rowKey, cfa, qualAvro, avroData));
 
-    Result r = new Result(kvs);
+    Result r = Result.create(kvs);
 
     Put p = new Put(rowKey);
 
@@ -1123,7 +1125,7 @@ public class TestHBaseSerDe extends TestCase {
   private Properties createPropertiesForHiveAvroForwardEvolvedSchema() {
     Properties tbl = new Properties();
     tbl.setProperty("cola.avro.serialization.type", "avro");
-    tbl.setProperty("cola.avro." + AvroSerdeUtils.SCHEMA_LITERAL, RECORD_SCHEMA_EVOLVED);
+    tbl.setProperty("cola.avro." + AvroTableProperties.SCHEMA_LITERAL.getPropName(), RECORD_SCHEMA_EVOLVED);
     tbl.setProperty(HBaseSerDe.HBASE_COLUMNS_MAPPING, ":key,cola:avro");
     tbl.setProperty(HBaseSerDe.HBASE_AUTOGENERATE_STRUCT, "true");
 
@@ -1138,13 +1140,13 @@ public class TestHBaseSerDe extends TestCase {
     byte[] rowKey = Bytes.toBytes("test-row1");
 
     // Data
-    List<KeyValue> kvs = new ArrayList<KeyValue>();
+    List<Cell> kvs = new ArrayList<Cell>();
 
     byte[] avroData = getTestAvroBytesFromSchema(RECORD_SCHEMA_EVOLVED);
 
     kvs.add(new KeyValue(rowKey, cfa, qualAvro, avroData));
 
-    Result r = new Result(kvs);
+    Result r = Result.create(kvs);
 
     Put p = new Put(rowKey);
 
@@ -1168,7 +1170,7 @@ public class TestHBaseSerDe extends TestCase {
   private Properties createPropertiesForHiveAvroBackwardEvolvedSchema() {
     Properties tbl = new Properties();
     tbl.setProperty("cola.avro.serialization.type", "avro");
-    tbl.setProperty("cola.avro." + AvroSerdeUtils.SCHEMA_LITERAL, RECORD_SCHEMA);
+    tbl.setProperty("cola.avro." + AvroTableProperties.SCHEMA_LITERAL.getPropName(), RECORD_SCHEMA);
     tbl.setProperty(HBaseSerDe.HBASE_COLUMNS_MAPPING, ":key,cola:avro");
     tbl.setProperty(HBaseSerDe.HBASE_AUTOGENERATE_STRUCT, "true");
 
@@ -1183,13 +1185,13 @@ public class TestHBaseSerDe extends TestCase {
     byte[] rowKey = Bytes.toBytes("test-row1");
 
     // Data
-    List<KeyValue> kvs = new ArrayList<KeyValue>();
+    List<Cell> kvs = new ArrayList<Cell>();
 
     byte[] avroData = getTestAvroBytesFromClass1(1);
 
     kvs.add(new KeyValue(rowKey, cfa, qualAvro, avroData));
 
-    Result r = new Result(kvs);
+    Result r = Result.create(kvs);
 
     Put p = new Put(rowKey);
 
@@ -1234,13 +1236,13 @@ public class TestHBaseSerDe extends TestCase {
     byte[] rowKey = Bytes.toBytes("test-row1");
 
     // Data
-    List<KeyValue> kvs = new ArrayList<KeyValue>();
+    List<Cell> kvs = new ArrayList<Cell>();
 
     byte[] avroData = getTestAvroBytesFromSchema(RECORD_SCHEMA);
 
     kvs.add(new KeyValue(rowKey, cfa, qualAvro, avroData));
 
-    Result r = new Result(kvs);
+    Result r = Result.create(kvs);
 
     Put p = new Put(rowKey);
 
@@ -1283,7 +1285,7 @@ public class TestHBaseSerDe extends TestCase {
   private Properties createPropertiesForHiveAvroSchemaUrl(String schemaUrl) {
     Properties tbl = new Properties();
     tbl.setProperty("cola.avro.serialization.type", "avro");
-    tbl.setProperty("cola.avro." + AvroSerdeUtils.SCHEMA_URL, schemaUrl);
+    tbl.setProperty("cola.avro." + AvroTableProperties.SCHEMA_URL.getPropName(), schemaUrl);
     tbl.setProperty(HBaseSerDe.HBASE_COLUMNS_MAPPING, ":key,cola:avro");
     tbl.setProperty(HBaseSerDe.HBASE_AUTOGENERATE_STRUCT, "true");
 
@@ -1298,13 +1300,13 @@ public class TestHBaseSerDe extends TestCase {
     byte[] rowKey = Bytes.toBytes("test-row1");
 
     // Data
-    List<KeyValue> kvs = new ArrayList<KeyValue>();
+    List<Cell> kvs = new ArrayList<Cell>();
 
     byte[] avroData = getTestAvroBytesFromClass2(1);
 
     kvs.add(new KeyValue(rowKey, cfa, qualAvro, avroData));
 
-    Result r = new Result(kvs);
+    Result r = Result.create(kvs);
 
     Put p = new Put(rowKey);
 
@@ -1333,7 +1335,7 @@ public class TestHBaseSerDe extends TestCase {
   private Properties createPropertiesForHiveAvroExternalSchema() {
     Properties tbl = new Properties();
     tbl.setProperty("cola.avro.serialization.type", "avro");
-    tbl.setProperty(AvroSerdeUtils.SCHEMA_RETRIEVER,
+    tbl.setProperty(AvroTableProperties.SCHEMA_RETRIEVER.getPropName(),
         "org.apache.hadoop.hive.hbase.HBaseTestAvroSchemaRetriever");
     tbl.setProperty("cola.avro." + serdeConstants.SERIALIZATION_CLASS,
         "org.apache.hadoop.hive.hbase.avro.Employee");
@@ -1362,7 +1364,7 @@ public class TestHBaseSerDe extends TestCase {
     byte[] rowKey = Bytes.toBytes("test-row1");
 
     // Data
-    List<KeyValue> kvs = new ArrayList<KeyValue>();
+    List<Cell> kvs = new ArrayList<Cell>();
 
     byte[] avroDataA = getTestAvroBytesFromSchema(RECORD_SCHEMA);
     byte[] avroDataB = getTestAvroBytesFromClass1(1);
@@ -1372,7 +1374,7 @@ public class TestHBaseSerDe extends TestCase {
     kvs.add(new KeyValue(rowKey, cfa, qualAvroB, avroDataB));
     kvs.add(new KeyValue(rowKey, cfa, qualAvroC, avroDataC));
 
-    Result r = new Result(kvs);
+    Result r = Result.create(kvs);
 
     Put p = new Put(rowKey);
 
@@ -1426,12 +1428,12 @@ public class TestHBaseSerDe extends TestCase {
     TestStruct testStruct = new TestStruct("A", "B", "C", false, (byte) 0);
     byte[] key = testStruct.getBytes();
     // Data
-    List<KeyValue> kvs = new ArrayList<KeyValue>();
+    List<Cell> kvs = new ArrayList<Cell>();
 
     byte[] testData = testStruct.getBytes();
     kvs.add(new KeyValue(key, cfa, qualStruct, testData));
 
-    Result r = new Result(kvs);
+    Result r = Result.create(kvs);
     byte[] putKey = testStruct.getBytesWithDelimiters();
 
     Put p = new Put(putKey);
@@ -1451,6 +1453,18 @@ public class TestHBaseSerDe extends TestCase {
 
   }
 
+  /**
+   * Since there are assertions in the code, when running this test it throws an assertion error
+   * and not the error in a production setup. The Properties.java object that is passed to the serDe
+   * initializer, is passed with empty value "" for "columns.comments" key for hbase backed tables.
+   */
+  public void testEmptyColumnComment() throws SerDeException {
+    HBaseSerDe serDe = new HBaseSerDe();
+    Properties tbl = createPropertiesForValueStruct();
+    tbl.setProperty("columns.comments", "");
+    serDe.initialize(new Configuration(), tbl);
+  }
+
   private Properties createPropertiesForValueStruct() {
     Properties tbl = new Properties();
     tbl.setProperty("cola.struct.serialization.type", "struct");
